<?php

declare(strict_types=1);

namespace Drupal\message_recipient_comment\Plugin\MessageRecipient\RecipientsCollector;

use Drupal\comment\CommentInterface;
use Drupal\comment\CommentManagerInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\message\MessageInterface;
use Drupal\message\MessageTemplateInterface;
use Drupal\message_recipient\Attribute\RecipientsCollector;
use Drupal\message_recipient_entity\ContentEntityUserRecipientsCollectorBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Commenters recipients collector.
 */
#[RecipientsCollector(
  id: 'commenters',
  type: 'user',
  category: 'comment',
  allow_multiple: FALSE,
  label: new TranslatableMarkup('Commenters'),
  description: new TranslatableMarkup("Collects commenters from a commentable entity by having a commentable entity or a comment as message's associated entity."),
)]
class Commenters extends ContentEntityUserRecipientsCollectorBase {

  /**
   * The comment manager.
   *
   * @var \Drupal\comment\CommentManagerInterface
   */
  protected CommentManagerInterface $commentManager;

  /**
   * The commenter ids.
   *
   * @var array
   */
  protected array $commenterIds = [];

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->commentManager = $container->get('comment.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function collect(MessageInterface $message): array {
    // Collects comments from each comment field.
    $entity = $this->getEntity($message);
    foreach ($this->configuration['comment_fields'][$entity->getEntityTypeId()] as $comment_field_name) {
      /** @var \Drupal\comment\CommentStorageInterface $storage */
      $storage = $this->entityTypeManager->getStorage('comment');

      $comments = $storage->loadThread($entity, $comment_field_name, CommentManagerInterface::COMMENT_MODE_FLAT);
      /** @var \Drupal\comment\CommentInterface $comment */
      foreach ($comments as $comment) {
        if (!in_array($comment->getOwnerId(), $this->commenterIds)) {
          $this->commenterIds[] = $comment->getOwnerId();
        }
      }
    }
    return !empty($this->commenterIds)
      ? array_values($this->query($message)->execute())
      : [];
  }

  /**
   * {@inheritdoc}
   */
  public function query(MessageInterface $message): QueryInterface {
    $query = parent::query($message);
    $query->condition('uid', $this->commenterIds, 'IN');
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(MessageInterface $message): bool {
    if ($this->hasAssociatedEntityField($message)) {
      $associated_entity_comment_fields = [];
      $associated_entity = $this->getAssociatedEntity($message);

      // Return true if the associated entity is a comment.
      if ($associated_entity instanceof CommentInterface) {
        return TRUE;
      }

      // Check if the associated entity is a commentable entity and make sure
      // the configuration matches with the comment field in the associated
      // entity.
      $comment_fields = $this->commentManager->getFields($associated_entity->getEntityTypeId());
      foreach ($comment_fields as $field_name => $info) {
        if ($associated_entity->hasField($field_name)
          && !$associated_entity->get($field_name)->isEmpty()) {
          $associated_entity_comment_fields[] = $field_name;
        }
      }
      return !empty($associated_entity_comment_fields)
        && !empty($this->configuration['comment_fields'][$associated_entity->getEntityTypeId()]);
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'comment_fields' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state, MessageTemplateInterface $message_template): array {
    $form['comment_fields'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type_id => $definition) {
      if (!$definition instanceof ContentEntityTypeInterface) {
        continue;
      }

      $comment_fields = [];
      $entity_type_comment_fields = $this->commentManager->getFields($entity_type_id);
      foreach ($entity_type_comment_fields as $field_name => $info) {
        $comment_fields[$field_name] = $this->t('Comment field %name in %bundles', [
          '%name' => $field_name,
          '%bundles' => implode(', ', $info['bundles']),
        ]);

        $form['comment_fields'][$entity_type_id] = [
          '#type' => 'checkboxes',
          '#title' => $this->t('Comment fields in %entity_type entity type:', ['%entity_type' => $definition->getLabel()]),
          '#default_value' => $this->configuration['comment_fields'][$entity_type_id] ?? [],
          '#options' => $comment_fields,
        ];
      }
    }

    return $form + parent::buildConfigurationForm($form, $form_state, $message_template);
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void {
    parent::validateConfigurationForm($form, $form_state, $message_template);
    if (empty($this->prepareFields($form_state->getValue('comment_fields')))) {
      $form_state->setErrorByName('', $this->t('You should select at least one field.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void {
    parent::submitConfigurationForm($form, $form_state, $message_template);
    $this->configuration['comment_fields'] = $this->prepareFields($form_state->getValue('comment_fields'));
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    return [
      'recipient_collector' => [
        '#markup' => $this->description(),
      ],
    ] + parent::getSummary();
  }

  /**
   * Prepare comment fields for saving.
   *
   * @param array $unfiltered_fields
   *   The comment fields grouped by entity types.
   *
   * @return array
   *   List of comment fields machine names grouped by entity types.
   */
  protected function prepareFields(array $unfiltered_fields): array {
    $fields = [];
    foreach ($unfiltered_fields as $entity_type => $entity_type_fields) {
      $filtered_fields = array_values(array_filter($entity_type_fields));
      if (!empty($filtered_fields)) {
        $fields[$entity_type] = $filtered_fields;
      }
    }
    return $fields;
  }

  /**
   * Get the commentable entity.
   *
   * @param \Drupal\message\MessageInterface $message
   *   The message entity.
   *
   * @return \Drupal\core\Entity\ContentEntityInterface|null
   *   The commentable entity if found, null otherwise.
   */
  protected function getEntity(MessageInterface $message): ?ContentEntityInterface {
    $associated_entity = $this->getAssociatedEntity($message);
    return match (TRUE) {
      $associated_entity instanceof commentInterface => $associated_entity->getCommentedEntity(),
      $associated_entity instanceof ContentEntityInterface => $associated_entity,
      default => NULL,
    };
  }

}
