<?php

declare(strict_types=1);

namespace Drupal\message_recipient_entity\Plugin\MessageRecipient\CollectorExtender;

use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\message\MessageInterface;
use Drupal\message_recipient\Attribute\CollectorExtender;
use Drupal\message_recipient\CollectorExtenderBase;
use Drupal\message_recipient\CollectorExtenderInterface;
use Drupal\message_recipient\CollectorExtenderType;
use Drupal\message_recipient\RecipientsCollectorInterface;
use Drupal\message_recipient_entity\MessageAssociatedEntityTrait;
use Drupal\user\EntityOwnerInterface;

/**
 * Include blocked user collector extender.
 */
#[CollectorExtender(
  id: 'exclude_owner',
  stage: CollectorExtenderType::QUERY_ALTER,
  types: ['user'],
  label: new TranslatableMarkup('Exclude owner'),
  description: new TranslatableMarkup("Exclude message's associated entity owner if collected by this recipients collector."),
)]
class ExcludeOwner extends CollectorExtenderBase implements CollectorExtenderInterface {

  use MessageAssociatedEntityTrait;

  /**
   * {@inheritdoc}
   */
  public function queryAlter(QueryInterface &$query, MessageInterface $message): void {
    if ($this->configuration['exclude_owner'] !== FALSE) {
      $query->condition('uid', $this->getAssociatedEntity($message)->getOwnerId(), '<>');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(MessageInterface $message): bool {
    if (parent::isApplicable($message) && $this->hasAssociatedEntityField($message)) {
      $associated_entity = $this->getAssociatedEntity($message);
      return $associated_entity instanceof EntityOwnerInterface;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'exclude_owner' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state, RecipientsCollectorInterface $recipients_collector): array {
    $form['exclude_owner'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Exclude owner'),
      '#default_value' => $this->configuration['exclude_owner'],
      '#description' => $this->t("When this option is selected, the user who owns the message's associated entity will be filtered out from the recipients collected by this collector. Use only if the associated entity has owner."),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state, RecipientsCollectorInterface $recipients_collector): void {
    $this->configuration['exclude_owner'] = (bool) $form_state->getValue('exclude_owner');
    parent::submitConfigurationForm($form, $form_state, $recipients_collector);
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    return $this->configuration['exclude_owner']
      ? ['#markup' => $this->label()]
      : [];
  }

}
