<?php

declare(strict_types=1);

namespace Drupal\message_recipient_entity\Plugin\MessageRecipient\RecipientsCollector;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\message\MessageInterface;
use Drupal\message\MessageTemplateInterface;
use Drupal\message_recipient\Attribute\RecipientsCollector;
use Drupal\message_recipient\MessageTemplateRecipientsCollectors;
use Drupal\message_recipient\RecipientsCollectorBase;
use Drupal\message_recipient_entity\MessageAssociatedEntityTrait;

/**
 * Email field recipients collector.
 */
#[RecipientsCollector(
  id: 'email_field',
  type: 'email',
  category: 'content entity',
  label: new TranslatableMarkup('Email field'),
  description: new TranslatableMarkup("Collects emails from the message's associated entity email field."),
)]
class EmailField extends RecipientsCollectorBase {

  use MessageAssociatedEntityTrait;

  /**
   * {@inheritdoc}
   */
  public function collect(MessageInterface $message): array {
    $associated_entity = $this->getAssociatedEntity($message);
    $emails = $associated_entity->get($this->configuration['field_name'])?->getValue();
    return array_column($emails, 'value');
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(MessageInterface $message): bool {
    if ($this->hasAssociatedEntityField($message)) {
      $associated_entity = $this->getAssociatedEntity($message);
      $field_name = $this->configuration['field_name'];
      if ($associated_entity->hasField($field_name)
        && !$associated_entity->get($field_name)->isEmpty()) {
        $field_definition = $associated_entity->getFieldDefinition($field_name);
        if ($field_definition->getType() === 'email') {
          return TRUE;
        }
      }
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'field_name' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state, MessageTemplateInterface $message_template): array {
    $form['field_name'] = [
      '#type' => 'machine_name',
      '#title' => $this->t('Email field machine name'),
      '#default_value' => $this->configuration['field_name'],
      '#required' => TRUE,
      '#machine_name' => [
        'exists' => [$this, 'exist'],
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void {
    // Validate that the message template does not have another email field
    // recipient collector using the same field.
    parent::validateConfigurationForm($form, $form_state, $message_template);
    if (!$this->getUuid()) {
      $recipients_collectors = clone (new MessageTemplateRecipientsCollectors($message_template))
        ->getRecipientsCollectors();
      /** @var \Drupal\message_recipient\RecipientsCollectorInterface $recipients_collector */
      foreach ($recipients_collectors as $recipients_collector) {
        if ($recipients_collector->getPluginId() !== $this->getPluginId()) {
          continue;
        }
        if (trim($form_state->getValue('field_name')) === $recipients_collector->getData()['field_name']) {
          $form_state->setErrorByName('field_name', $this->t('The message template has already "%label" recipient collector using "%field_name" field', [
            '%label' => $this->label(),
            '%field_name' => $form_state->getValue('field_name'),
          ]));
          break;
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void {
    parent::submitConfigurationForm($form, $form_state, $message_template);
    $this->configuration['field_name'] = trim($form_state->getValue('field_name'));
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    return [
      'recipient_collector' => [
        '#markup' => $this->t("Collects emails from the message's associated entity %field_name field.", [
          '%field_name' => $this->configuration['field_name'] ?: '',
        ]),
      ],
    ] + parent::getSummary();
  }

  /**
   * Callback method for machine name field.
   *
   * @return false
   *   Always return false.
   */
  public function exist(): FALSE {
    return FALSE;
  }

}
