<?php

declare(strict_types=1);

namespace Drupal\message_recipient_ui\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\message\MessageTemplateInterface;
use Drupal\message_recipient\MessageTemplateRecipientsCollectors;
use Drupal\message_recipient\RecipientsCollectorInterface;
use Drupal\message_recipient\RecipientsCollectorManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form for deleting a recipients collector.
 */
class RecipientsCollectorDeleteForm extends ConfirmFormBase {

  /**
   * The message template containing the recipients' collector to be deleted.
   *
   * @var \Drupal\message\MessageTemplateInterface
   */
  protected MessageTemplateInterface $messageTemplate;

  /**
   * The recipients' collector.
   *
   * @var \Drupal\message_recipient\RecipientsCollectorInterface
   */
  protected RecipientsCollectorInterface $recipientsCollector;

  /**
   * The recipients collector manager.
   *
   * @var \Drupal\message_recipient\RecipientsCollectorManagerInterface
   */
  protected RecipientsCollectorManagerInterface $recipientsCollectorManager;

  /**
   * Constructs a new RecipientsCollectorDeleteForm.
   *
   * @param \Drupal\message_recipient\RecipientsCollectorManagerInterface $recipients_collector_manager
   *   The recipients collector manager.
   */
  public function __construct(RecipientsCollectorManagerInterface $recipients_collector_manager) {
    $this->recipientsCollectorManager = $recipients_collector_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('plugin.manager.message_recipient.recipients_collector')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion(): TranslatableMarkup {
    return $this->t('Are you sure you want to delete the "@recipient_collector" recipients collector from the "%message_template" message template?', [
      '@recipient_collector' => $this->recipientsCollector->label(),
      '%message_template' => $this->messageTemplate->label(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText(): TranslatableMarkup {
    return $this->t('Delete');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl(): Url {
    return new Url('message_recipient_ui.message_template.recipients_collectors', [
      'message_template' => $this->messageTemplate->id(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'recipients_collector_delete_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, MessageTemplateInterface $message_template = NULL, string $recipients_collector = ''): array {
    $this->messageTemplate = $message_template;
    $this->recipientsCollector = (new MessageTemplateRecipientsCollectors($this->messageTemplate))->getRecipientsCollector($recipients_collector);

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    (new MessageTemplateRecipientsCollectors($this->messageTemplate))
      ->deleteRecipientsCollector($this->recipientsCollector->getUuid());
    $this->messenger()->addStatus($this->t('The recipients collector %name has been deleted.', ['%name' => $this->recipientsCollector->label()]));
    $form_state->setRedirectUrl(
      new Url('message_recipient_ui.message_template.recipients_collectors', [
        'message_template' => $this->messageTemplate->id(),
      ])
    );
  }

}
