<?php

declare(strict_types=1);

namespace Drupal\message_recipient_ui\Form;

use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Url;
use Drupal\message\MessageTemplateInterface;
use Drupal\message_recipient\CollectorExtenderManagerInterface;
use Drupal\message_recipient\RecipientsCollectorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Base form for recipients' collector.
 */
abstract class RecipientsCollectorFormBase extends FormBase {

  /**
   * The message template.
   *
   * @var \Drupal\message\MessageTemplateInterface
   */
  protected MessageTemplateInterface $messageTemplate;

  /**
   * The recipients' collector.
   *
   * @var \Drupal\message_recipient\RecipientsCollectorInterface
   */
  protected RecipientsCollectorInterface $recipientsCollector;

  /**
   * The collector extender manager.
   *
   * @var \Drupal\message_recipient\CollectorExtenderManagerInterface
   */
  protected CollectorExtenderManagerInterface $collectorExtenderManager;

  /**
   * Constructs OnboardingStepFormBase object.
   *
   * @param \Drupal\message_recipient\CollectorExtenderManagerInterface $collector_extender_manager
   *   The collector extender manager.
   */
  public function __construct(CollectorExtenderManagerInterface $collector_extender_manager) {
    $this->collectorExtenderManager = $collector_extender_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('plugin.manager.message_recipient.collector_extender'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'recipients_collector_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, MessageTemplateInterface $message_template = NULL, string $recipients_collector = NULL): array {
    $this->messageTemplate = $message_template;
    try {
      $this->recipientsCollector = $this->prepareRecipientsCollector($recipients_collector);
    }
    catch (PluginNotFoundException) {
      throw new NotFoundHttpException("Invalid recipients collector id: '$recipients_collector'.");
    }

    $form['id'] = [
      '#type' => 'value',
      '#value' => $this->recipientsCollector->getPluginId(),
    ];

    // Build the configuration form.
    $form['data'] = [];
    $subform_state = SubformState::createForSubform($form['data'], $form, $form_state);
    $form['data'] = $this->recipientsCollector->buildConfigurationForm($form['data'], $subform_state, $this->messageTemplate);
    $form['data']['#tree'] = TRUE;

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#button_type' => 'primary',
    ];
    $form['actions']['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => new Url('message_recipient_ui.message_template.recipients_collectors', [
        'message_template' => $this->messageTemplate->id(),
      ]),
      '#attributes' => ['class' => ['button']],
    ];

    return $form + $this->collectorExtendersForm($form, $form_state, $message_template);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // The recipients' collector configuration is stored in the 'data' key in
    // the form, pass that through for validation.
    $this->recipientsCollector->validateConfigurationForm($form['data'], SubformState::createForSubform($form['data'], $form, $form_state), $this->messageTemplate);

    // Validate extenders forms.
    $extenders = $this->getAllExtenders();
    foreach ($extenders as $extender_id => $extender) {
      $extender_form_state = SubformState::createForSubform($form['extenders']['settings'][$extender_id], $form, $form_state);
      $extender->validateConfigurationForm($form['extenders']['settings'][$extender_id], $extender_form_state, $this->recipientsCollector);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $form_state->cleanValues();

    // The recipients' collector configuration is stored in the 'data' key in
    // the form, pass that through for submission.
    $this->recipientsCollector->submitConfigurationForm($form['data'], SubformState::createForSubform($form['data'], $form, $form_state), $this->messageTemplate);

    // Store extenders settings.
    $extenders = $this->getAllExtenders();
    foreach ($extenders as $extender_id => $extender) {
      $extender_form_state = SubformState::createForSubform($form['extenders']['settings'][$extender_id], $form, $form_state);
      $extender->submitConfigurationForm($form['extenders']['settings'][$extender_id], $extender_form_state, $this->recipientsCollector);
    }
  }

  /**
   * Converts a recipients' collector ID into an object.
   *
   * @param string $recipients_collector_id
   *   The recipients' collector ID.
   *
   * @return \Drupal\message_recipient\RecipientsCollectorInterface
   *   The recipients' collector object.
   */
  abstract protected function prepareRecipientsCollector(string $recipients_collector_id): RecipientsCollectorInterface;

  /**
   * Extenders form constructor.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function collectorExtendersForm(array $form, FormStateInterface $form_state): array {
    $extenders = $this->getAllExtenders();

    if (empty($extenders)) {
      return $form;
    }

    $form['extenders'] = [
      '#type' => 'container',
      '#tree' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="extenders_description"]' => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];

    // Add vertical tabs containing the settings for the extenders.
    $form['extenders']['extender_settings'] = [
      '#title' => $this->t('Collector extender settings'),
      '#type' => 'vertical_tabs',
      '#description' => $this->t('Configure the collector extenders that will alter the recipients list.'),
      '#description_display' => 'before',
    ];

    foreach ($extenders as $extender_id => $extender) {
      $form['extenders']['settings'][$extender_id] = [
        '#type' => 'details',
        '#title' => $extender->label(),
        '#group' => 'extenders][extender_settings',
        '#parents' => ['extenders', $extender_id, 'settings'],
      ];

      $extender_form_state = SubformState::createForSubform($form['extenders']['settings'][$extender_id], $form, $form_state);
      $form['extenders']['settings'][$extender_id] += $extender->buildConfigurationForm($form['extenders']['settings'][$extender_id], $extender_form_state, $this->recipientsCollector);
    }

    return $form;
  }

  /**
   * Returns all enabled and not enabled extenders.
   *
   * @return array
   *   The extenders.
   */
  public function getAllExtenders(): array {
    $extenders = [];
    $enabled_extenders = $this->recipientsCollector->getExtenders();

    foreach ($this->collectorExtenderManager->getDefinitions() as $name => $extender_definition) {
      if ($enabled_extenders->has($name)) {
        $extenders[$name] = $enabled_extenders->get($name);
        continue;
      }

      /** @var \Drupal\message_recipient\CollectorExtenderInterface $extender */
      $extender = $this->collectorExtenderManager->createInstance($name);
      if (!empty($extender->getTypes()) && !in_array($this->recipientsCollector->getType(), $extender->getTypes())) {
        continue;
      }
      $extenders[$name] = $extender;
    }

    return $extenders;
  }

}
