<?php

declare(strict_types=1);

namespace Drupal\message_recipient_ui\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\message\MessageTemplateInterface;
use Drupal\message_recipient\MessageTemplateRecipientsCollectors;
use Drupal\message_recipient\RecipientsCollectorManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form used to add and manage recipients' collectors for a message template.
 */
class RecipientsCollectorsForm extends FormBase {

  /**
   * The recipients collector manager.
   *
   * @var \Drupal\message_recipient\RecipientsCollectorManagerInterface
   */
  protected RecipientsCollectorManagerInterface $recipientsCollectorManager;

  /**
   * The message template containing the recipients collectors.
   *
   * @var \Drupal\message\MessageTemplateInterface
   */
  protected MessageTemplateInterface $messageTemplate;

  /**
   * Constructs a new RecipientsCollectorsForm.
   *
   * @param \Drupal\message_recipient\RecipientsCollectorManagerInterface $recipients_collector_manager
   *   The recipients collector manager.
   */
  public function __construct(RecipientsCollectorManagerInterface $recipients_collector_manager) {
    $this->recipientsCollectorManager = $recipients_collector_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('plugin.manager.message_recipient.recipients_collector')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'recipients_collectors_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, MessageTemplateInterface $message_template = NULL): array {
    $form['#attached']['library'][] = 'message_recipient_ui/admin';
    $this->messageTemplate = $message_template;
    // Build the list of existing recipients collectors for the message
    // template.
    $form['recipients_collectors'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Recipients collector'),
        $this->t('Summary'),
        $this->t('Operations'),
      ],
      '#empty' => $this->t('There are currently no recipients collectors in this message template. Add one by selecting an option below.'),
    ];

    // Prepare the list of the existing collectors.
    $recipients_collectors = (new MessageTemplateRecipientsCollectors($this->messageTemplate))
      ->getRecipientsCollectors();
    /** @var \Drupal\message_recipient\RecipientsCollectorInterface $recipients_collector */
    foreach ($recipients_collectors as $recipients_collector) {
      $key = $recipients_collector->getUuid();

      // Label.
      $form['recipients_collectors'][$key]['plugin'] = [
        '#tree' => FALSE,
        'data' => [
          'label' => [
            '#plain_text' => $recipients_collector->label(),
          ],
        ],
      ];

      // Summary.
      $form['recipients_collectors'][$key]['summary'] = [
        '#tree' => FALSE,
        'data' => [
          'summary' => [
            $recipients_collector->getSummary(),
          ],
        ],
      ];

      // Operations.
      $links = [];
      $links['edit'] = [
        'title' => $this->t('Edit'),
        'url' => Url::fromRoute('message_recipient_ui.message_template.recipients_collector.edit_form', [
          'message_template' => $message_template->id(),
          'recipients_collector' => $key,
        ]),
      ];
      $links['delete'] = [
        'title' => $this->t('Delete'),
        'url' => Url::fromRoute('message_recipient_ui.message_template.recipients_collector.delete_form', [
          'message_template' => $message_template->id(),
          'recipients_collector' => $key,
        ]),
      ];
      $form['recipients_collectors'][$key]['operations'] = [
        '#type' => 'operations',
        '#links' => $links,
      ];
    }

    // Build the new recipient plugin addition form and add it to the list.
    $form['recipients_collectors']['new'] = [
      '#tree' => FALSE,
    ];
    $form['recipients_collectors']['new']['collector'] = [
      'data' => [
        'new' => [
          '#type' => 'select',
          '#title' => $this->t('New recipients collector'),
          '#title_display' => 'invisible',
          '#options' => $this->recipientsCollectorManager->getDefinitionsListOptionsGroupedByCategory(),
          '#empty_option' => $this->t('Select a new recipients collector'),
        ],
        [
          'add' => [
            '#type' => 'submit',
            '#value' => $this->t('Add'),
            '#validate' => ['::recipientsCollectorValidate'],
            '#submit' => ['::submitForm', '::recipientsCollectorSave'],
          ],
        ],
      ],
      '#prefix' => '<div class="recipients-collector-new">',
      '#suffix' => '</div>',
    ];
    $form['recipients_collectors']['new']['summary'] = [
      'data' => [],
    ];
    $form['recipients_collectors']['new']['operations'] = [
      'data' => [],
    ];
    return $form;
  }

  /**
   * Validate handler for recipients collector.
   */
  public function recipientsCollectorValidate($form, FormStateInterface $form_state): void {
    $new_recipients_collector_id = $form_state->getValue('new');
    if (!$new_recipients_collector_id) {
      $form_state->setErrorByName('new', $this->t('Select a recipients collector to add.'));
    }
    else {
      $has_recipients_collector = (new MessageTemplateRecipientsCollectors($this->messageTemplate))
        ->hasRecipientsCollector($new_recipients_collector_id);
      if ($has_recipients_collector) {
        $recipients_collector_definition = $this->recipientsCollectorManager->getDefinitionById($new_recipients_collector_id);
        if (!$recipients_collector_definition['allow_multiple']) {
          $form_state->setErrorByName('new', $this->t('The select recipients collector is already exist.'));
        }
      }
    }
  }

  /**
   * Submit handler for recipients' collector.
   */
  public function recipientsCollectorSave($form, FormStateInterface $form_state): void {
    $form_state->setRedirect(
      'message_recipient_ui.message_template.recipients_collector.add_form', [
        'message_template' => $this->messageTemplate->id(),
        'recipients_collector' => $form_state->getValue('new'),
      ]
    );
    $form_state->setIgnoreDestination();
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
  }

  /**
   * Form title callback.
   *
   * @param \Drupal\message\MessageTemplateInterface $message_template
   *   The message template.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The form title.
   */
  public function formTitle(MessageTemplateInterface $message_template): TranslatableMarkup {
    return $this->t('Recipients collectors for: %name', ['%name' => $message_template->label()]);
  }

}
