<?php

declare(strict_types=1);

namespace Drupal\message_recipient;

use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\message\MessageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for collector extender instances.
 */
abstract class CollectorExtenderBase extends PluginBase implements CollectorExtenderInterface {

  /**
   * The recipients' collector that stores the extender.
   *
   * @var \Drupal\message_recipient\RecipientsCollectorInterface
   */
  protected RecipientsCollectorInterface $recipientsCollector;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->setConfiguration($configuration);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
    );
  }

  /**
   * {@inheritdoc}
   */
  public function initialize(RecipientsCollectorInterface $recipients_collector): void {
    $this->recipientsCollector = $recipients_collector;
  }

  /**
   * {@inheritdoc}
   */
  public function label(): TranslatableMarkup {
    return $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function description(): TranslatableMarkup {
    return $this->pluginDefinition['description'];
  }

  /**
   * {@inheritdoc}
   */
  public function getStage(): CollectorExtenderType {
    return $this->pluginDefinition['stage'];
  }

  /**
   * {@inheritdoc}
   */
  public function getTypes(): array {
    return $this->pluginDefinition['types'];
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(MessageInterface $message): bool {
    if (!empty($this->getTypes())) {
      return in_array($this->recipientsCollector->getType(), $this->getTypes());
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration(): array {
    return [
      'id' => $this->getPluginId(),
      'settings' => $this->configuration,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration): self {
    $configuration += [
      'settings' => [],
    ];
    $this->configuration = $configuration['settings'] + $this->defaultConfiguration();
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getSettings(): array {
    return $this->getConfiguration()['settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    return [
      '#markup' => $this->pluginDefinition['description'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function queryAlter(QueryInterface &$query, MessageInterface $message): void {
    // Nothing to do. Child class of stage 'query_alter' should implement its
    // alter for the query in this method.
  }

  /**
   * {@inheritdoc}
   */
  public function postCollect(array &$recipients, MessageInterface $message): void {
    // Nothing to do. Child class of stage 'post_collect' should implement its
    // post-collect functionality in this method.
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state, RecipientsCollectorInterface $recipients_collector): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state, RecipientsCollectorInterface $recipients_collector): void {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state, RecipientsCollectorInterface $recipients_collector): void {
    $recipients_collector->addExtender($this->pluginId, $this->configuration);
  }

}
