<?php

declare(strict_types=1);

namespace Drupal\message_recipient;

use Drupal\Component\Plugin\ConfigurableInterface;
use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\message\MessageInterface;

/**
 * Interface for recipients collector instance.
 */
interface CollectorExtenderInterface extends PluginInspectionInterface, ConfigurableInterface, ContainerFactoryPluginInterface {

  /**
   * Set recipients' collector when initializing a collector extender plugin.
   *
   * @param \Drupal\message_recipient\RecipientsCollectorInterface $recipients_collector
   *   The message template.
   *
   * @see \Drupal\message_recipient\CollectorExtenderPluginCollection
   */
  public function initialize(RecipientsCollectorInterface $recipients_collector): void;

  /**
   * Returns the collector extender label.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The collector extender label.
   */
  public function label(): TranslatableMarkup;

  /**
   * Returns the collector extender description.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The collector extender description.
   */
  public function description(): TranslatableMarkup;

  /**
   * Returns the collector extender stage.
   *
   * @return \Drupal\message_recipient\CollectorExtenderType
   *   The collector extender stage.
   */
  public function getStage(): CollectorExtenderType;

  /**
   * Returns the collector extender types.
   *
   * @return array
   *   The collector extender types.
   */
  public function getTypes(): array;

  /**
   * Check if whether the collector extender is applicable.
   *
   * @param \Drupal\message\MessageInterface $message
   *   The message entity.
   *
   * @return bool
   *   True if the collector extender is applicable, false otherwise.
   */
  public function isApplicable(MessageInterface $message): bool;

  /**
   * Get settings from configuration.
   *
   * @return array
   *   Settings from configuration.
   */
  public function getSettings(): array;

  /**
   * Returns a render array summarizing the configuration.
   *
   * @return array
   *   A render array.
   */
  public function getSummary(): array;

  /**
   * Alters a recipients' collector query.
   *
   * @param \Drupal\Core\Entity\Query\QueryInterface $query
   *   The recipients' collector query to alter.
   * @param \Drupal\message\MessageInterface $message
   *   The message entity.
   */
  public function queryAlter(QueryInterface &$query, MessageInterface $message): void;

  /**
   * Post-collect.
   *
   * @param array $recipients
   *   The array of recipients.
   * @param \Drupal\message\MessageInterface $message
   *   The message entity.
   */
  public function postCollect(array &$recipients, MessageInterface $message): void;

  /**
   * Adds a configuration form for this collector extender.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param \Drupal\message_recipient\RecipientsCollectorInterface $recipients_collector
   *   The recipients' collector this collector extender is being added to.
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state, RecipientsCollectorInterface $recipients_collector): array;

  /**
   * Validates a configuration form for this collector extender.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param \Drupal\message_recipient\RecipientsCollectorInterface $recipients_collector
   *   The recipients' collector this collector extender is being added to.
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state, RecipientsCollectorInterface $recipients_collector): void;

  /**
   * Submits a configuration form for this collector extender.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param \Drupal\message_recipient\RecipientsCollectorInterface $recipients_collector
   *   The recipients' collector this collector extender is being added to.
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state, RecipientsCollectorInterface $recipients_collector): void;

}
