<?php

declare(strict_types=1);

namespace Drupal\message_recipient;

use Drupal\message\MessageTemplateInterface;

/**
 * Class to manage the recipients' collector for a message template.
 */
class MessageTemplateRecipientsCollectors {

  /**
   * The message template.
   *
   * @var \Drupal\message\MessageTemplateInterface
   */
  protected MessageTemplateInterface $messageTemplate;

  /**
   * Holds the collection of recipients' collectors used by a message template.
   *
   * @var \Drupal\message_recipient\RecipientsCollectorPluginCollection|null
   */
  protected ?RecipientsCollectorPluginCollection $recipientsCollectorsCollection = NULL;

  /**
   * Constructs a new MessageTemplateRecipientsCollectors object.
   */
  public function __construct(MessageTemplateInterface $message_template) {
    $this->messageTemplate = $message_template;
  }

  /**
   * Check whether the message template has specific recipients' collector.
   *
   * @param string $recipients_collector_id
   *   The recipients collector id.
   *
   * @return bool
   *   True if the message template has the recipients' collector, false
   *   otherwise.
   */
  public function hasRecipientsCollector(string $recipients_collector_id): bool {
    $recipients_collectors = $this->getRecipientsCollectors();
    /** @var \Drupal\message_recipient\RecipientsCollectorInterface $recipients_collector */
    foreach ($recipients_collectors as $recipients_collector) {
      if ($recipients_collector->getPluginId() === $recipients_collector_id) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Get a specific recipients' collector from the message template by id.
   *
   * @param string $recipients_collector_id
   *   The recipients collector id.
   *
   * @return \Drupal\message_recipient\RecipientsCollectorInterface
   *   The recipients' collector.
   */
  public function getRecipientsCollector(string $recipients_collector_id): RecipientsCollectorInterface {
    return $this->getRecipientsCollectors()->get($recipients_collector_id);
  }

  /**
   * Get a collection of all recipients' collectors from message template.
   *
   * @return \Drupal\message_recipient\RecipientsCollectorPluginCollection|null
   *   The recipients' collector, if any, null otherwise.
   */
  public function getRecipientsCollectors(): ?RecipientsCollectorPluginCollection {
    if (!$this->recipientsCollectorsCollection) {
      $this->recipientsCollectorsCollection = new RecipientsCollectorPluginCollection(\Drupal::service('plugin.manager.message_recipient.recipients_collector'), $this->messageTemplate, $this->getSettings());
      $this->recipientsCollectorsCollection->sort();
    }
    return $this->recipientsCollectorsCollection;
  }

  /**
   * Add new recipients collector to the message template.
   *
   * @param string $recipients_collector_id
   *   The recipients collector id.
   * @param array $configuration
   *   The configuration for the recipients' collector.
   */
  public function addRecipientsCollector(string $recipients_collector_id, array $configuration = []): void {
    $recipients_collectors = $this->getSettings();
    $uuid = \Drupal::service('uuid')->generate();
    $recipients_collectors[$uuid] = [
      'id' => $recipients_collector_id,
      'uuid' => $uuid,
    ];
    if (!empty($configuration)) {
      // Get the default configuration.
      $recipients_collectors[$uuid] = $recipients_collectors[$uuid] + $configuration;
    }
    $this->messageTemplate->setThirdPartySetting('message_recipient', 'recipients_collectors', $recipients_collectors)
      ->save();
  }

  /**
   * Edit existing recipients' collector.
   *
   * @param string $recipients_collector_uuid
   *   The recipients collector uuid.
   * @param array $configuration
   *   The new configuration for the recipients' collector.
   */
  public function editRecipientsCollector(string $recipients_collector_uuid, array $configuration = []): void {
    $recipients_collectors = $this->getSettings();
    $recipients_collectors[$recipients_collector_uuid] = [
      'id' => $configuration['id'],
      'uuid' => $recipients_collector_uuid,
    ];
    $recipients_collectors[$recipients_collector_uuid] = $recipients_collectors[$recipients_collector_uuid] + $configuration;
    $this->messageTemplate->setThirdPartySetting('message_recipient', 'recipients_collectors', $recipients_collectors)
      ->save();
  }

  /**
   * Remove a recipients' collector from the message template.
   *
   * @param string $recipients_collector_uuid
   *   The recipients collector uuid.
   */
  public function deleteRecipientsCollector(string $recipients_collector_uuid): void {
    $recipients_collectors = $this->getSettings();
    unset($recipients_collectors[$recipients_collector_uuid]);
    $this->messageTemplate->setThirdPartySetting('message_recipient', 'recipients_collectors', $recipients_collectors)
      ->save();
  }

  /**
   * Get recipients collectors third party settings from the message template.
   *
   * @return array
   *   The setting for the recipients' collectors.
   */
  protected function getSettings(): array {
    return $this->messageTemplate->getThirdPartySetting('message_recipient', 'recipients_collectors') ?? [];
  }

}
