<?php

declare(strict_types=1);

namespace Drupal\message_recipient\Plugin\MessageRecipient\CollectorExtender;

use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\message\MessageInterface;
use Drupal\message_recipient\Attribute\CollectorExtender;
use Drupal\message_recipient\CollectorExtenderBase;
use Drupal\message_recipient\CollectorExtenderInterface;
use Drupal\message_recipient\CollectorExtenderType;
use Drupal\message_recipient\RecipientsCollectorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Include blocked user collector extender.
 */
#[CollectorExtender(
  id: 'include_actor',
  stage: CollectorExtenderType::QUERY_ALTER,
  types: ['user'],
  label: new TranslatableMarkup('Include actor'),
  description: new TranslatableMarkup('Includes actor if collected by this recipients collector.'),
)]
class IncludeActor extends CollectorExtenderBase implements CollectorExtenderInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected AccountInterface $currentUser;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->currentUser = $container->get('current_user');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function queryAlter(QueryInterface &$query, MessageInterface $message): void {
    if ($this->configuration['include_actor'] !== TRUE) {
      $query->condition('uid', $this->currentUser->id(), '<>');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'include_actor' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state, RecipientsCollectorInterface $recipients_collector): array {
    $form['include_actor'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Include actor'),
      '#default_value' => $this->configuration['include_actor'],
      '#description' => $this->t("When this option is selected, the user who initiated the message entity won't be filtered out from the recipients collected by this collector."),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state, RecipientsCollectorInterface $recipients_collector): void {
    $this->configuration['include_actor'] = (bool) $form_state->getValue('include_actor');
    parent::submitConfigurationForm($form, $form_state, $recipients_collector);
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    return $this->configuration['include_actor']
      ? ['#markup' => $this->label()]
      : [];
  }

}
