<?php

declare(strict_types=1);

namespace Drupal\message_recipient\Plugin\MessageRecipient\RecipientsCollector;

use Drupal\Component\Utility\EmailValidatorInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\message\MessageInterface;
use Drupal\message\MessageTemplateInterface;
use Drupal\message_recipient\Attribute\RecipientsCollector;
use Drupal\message_recipient\RecipientsCollectorBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Mailbox recipients collector.
 */
#[RecipientsCollector(
  id: 'mailbox',
  type: 'email',
  category: 'general',
  allow_multiple: FALSE,
  label: new TranslatableMarkup('Mailbox'),
  description: new TranslatableMarkup('Collects provided mailboxes.'),
)]
class Mailbox extends RecipientsCollectorBase {

  /**
   * The email validator.
   *
   * @var \Drupal\Component\Utility\EmailValidatorInterface
   */
  protected EmailValidatorInterface $emailValidator;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->emailValidator = $container->get('email.validator');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function collect(MessageInterface $message): array {
    return $this->configuration['mailboxes'];
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(MessageInterface $message): bool {
    return !empty($this->configuration['mailboxes']);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'mailboxes' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state, MessageTemplateInterface $message_template): array {
    $form['mailboxes'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Mailboxes'),
      '#description' => $this->t('Enter one email per line.'),
      '#default_value' => implode("\r\n", $this->configuration['mailboxes']),
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void {
    parent::validateConfigurationForm($form, $form_state, $message_template);
    foreach ($this->extractEmails($form_state->getValue('mailboxes')) as $email) {
      if (!$this->emailValidator->isValid($email)) {
        $form_state->setErrorByName('mailboxes', $this->t('%email is an invalid email address.', [
          '%email' => $email,
        ]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void {
    parent::submitConfigurationForm($form, $form_state, $message_template);
    $this->configuration['mailboxes'] = $this->extractEmails($form_state->getValue('mailboxes'));
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    return [
      'recipient_collector' => [
        '#markup' => $this->t('Collects provided mailboxes: %mailboxes', [
          '%mailboxes' => implode(', ', $this->configuration['mailboxes']),
        ]),
      ],
    ] + parent::getSummary();
  }

  /**
   * Extract email from string.
   *
   * @param string $emails
   *   Emails separated by new line.
   *
   * @return array
   *   The extracted emails in an array.
   */
  protected function extractEmails(string $emails): array {
    return explode("\r\n", $emails);
  }

}
