<?php

declare(strict_types=1);

namespace Drupal\message_recipient\Plugin\MessageRecipient\RecipientsCollector;

use Drupal\Component\Utility\Html;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\message\MessageInterface;
use Drupal\message\MessageTemplateInterface;
use Drupal\message_recipient\Attribute\RecipientsCollector;
use Drupal\message_recipient\UserRecipientsCollectorBase;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;

/**
 * Users by roles recipients' collector.
 */
#[RecipientsCollector(
  id: 'users_by_roles',
  type: 'user',
  category: 'general',
  label: new TranslatableMarkup('Users by roles'),
  description: new TranslatableMarkup('Collects users by roles.'),
)]
class UsersByRoles extends UserRecipientsCollectorBase {

  /**
   * {@inheritdoc}
   */
  public function collect(MessageInterface $message): array {
    return array_values($this->query($message)->execute());
  }

  /**
   * {@inheritdoc}
   */
  public function query(MessageInterface $message): QueryInterface {
    $query = parent::query($message);
    $query->condition('roles', $this->configuration['roles'], 'IN');
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(MessageInterface $message): bool {
    return !empty($this->configuration['roles']);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'roles' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state, MessageTemplateInterface $message_template): array {
    $form['roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#options' => $this->getRoles(),
      '#default_value' => $this->configuration['roles'],
      '#required' => TRUE,
    ];
    return $form + parent::buildConfigurationForm($form, $form_state, $message_template);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void {
    parent::submitConfigurationForm($form, $form_state, $message_template);
    $this->configuration['roles'] = array_values(array_filter($form_state->getValue('roles')));
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    return [
      'recipient_collector' => [
        '#markup' => $this->t('Collect users by the following roles: %roles', [
          '%roles' => implode(', ', $this->getRoles($this->configuration['roles'])),
        ]),
      ],
    ] + parent::getSummary();
  }

  /**
   * Get roles by ids.
   *
   * @param array|null $role_ids
   *   The roles ids to load.
   *
   * @return array
   *   List of roles labels keyed by role ids.
   */
  protected function getRoles(array $role_ids = NULL): array {
    $roles = array_map(
      fn(RoleInterface $role) => Html::escape($role->label()), Role::loadMultiple($role_ids)
    );
    unset($roles[RoleInterface::ANONYMOUS_ID]);
    return $roles;
  }

}
