<?php

declare(strict_types=1);

namespace Drupal\message_recipient;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\message\MessageInterface;
use Drupal\message\MessageTemplateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for recipients collector instances.
 */
abstract class RecipientsCollectorBase extends PluginBase implements RecipientsCollectorInterface {

  /**
   * The recipients' collector uuid.
   *
   * @var string
   */
  protected string $uuid;

  /**
   * The recipients' collector extenders.
   *
   * @var array
   */
  protected array $extenders;

  /**
   * The message template.
   *
   * @var \Drupal\message\MessageTemplateInterface
   */
  protected MessageTemplateInterface $messageTemplate;

  /**
   * The collector extender plugin collection.
   *
   * @var \Drupal\message_recipient\CollectorExtenderPluginCollection|null
   */
  protected ?CollectorExtenderPluginCollection $collectorExtendersCollection = NULL;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->setConfiguration($configuration);
    $this->setExtenders($configuration);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
    );
  }

  /**
   * {@inheritdoc}
   */
  public function initialize(MessageTemplateInterface $message_template): void {
    $this->messageTemplate = $message_template;
  }

  /**
   * {@inheritdoc}
   */
  public function getUuid(): string {
    return $this->uuid;
  }

  /**
   * {@inheritdoc}
   */
  public function label(): TranslatableMarkup {
    return $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function description(): TranslatableMarkup {
    return $this->pluginDefinition['description'];
  }

  /**
   * {@inheritdoc}
   */
  public function getType(): string {
    return $this->pluginDefinition['type'];
  }

  /**
   * {@inheritdoc}
   */
  public function getCategory(): string {
    return $this->getConfiguration()['category'];
  }

  /**
   * {@inheritdoc}
   */
  public function isMultipleAllowed(): bool {
    return $this->pluginDefinition['allow_multiple'];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration(): array {
    return [
      'uuid' => $this->getUuid(),
      'id' => $this->getPluginId(),
      'data' => $this->configuration,
      'extenders' => $this->extenders,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration): self {
    $configuration += [
      'uuid' => '',
      'data' => [],
    ];
    $this->uuid = $configuration['uuid'];
    $this->configuration = $configuration['data'] + $this->defaultConfiguration();

    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getData(): array {
    return $this->getConfiguration()['data'];
  }

  /**
   * {@inheritdoc}
   */
  public function setExtenders(array $configuration): self {
    $this->extenders = $configuration['extenders'] ?? [];
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function addExtender(string $extender_id, array $settings = []): self {
    if (!empty($settings)) {
      $this->extenders[$extender_id] = [
        'id' => $extender_id,
        'settings' => $settings,
      ];
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function removeExtender(string $extender_id): self {
    unset($this->extenders[$extender_id]);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getExtenders(): CollectorExtenderPluginCollection {
    if (!$this->collectorExtendersCollection) {
      $this->collectorExtendersCollection = new CollectorExtenderPluginCollection(\Drupal::service('plugin.manager.message_recipient.collector_extender'), $this, $this->extenders);
      $this->collectorExtendersCollection->sort();
    }
    return $this->collectorExtendersCollection;
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    $extenders = $this->getExtenders();
    $collector_extenders_summary = [];
    $summary = [];
    foreach ($extenders as $extender) {
      if (!empty($extender->getSummary())) {
        $summary[] = $extender->getSummary();
      }
    }
    if (!empty($summary)) {
      $collector_extenders_summary[] = [
        '#type' => 'html_tag',
        '#tag' => 'hr',
      ];
      $collector_extenders_summary[] = [
        '#type' => 'html_tag',
        '#tag' => 'strong',
        '#value' => $this->t('Configured collector extenders:'),
      ];
      $collector_extenders_summary[] = [
        '#theme' => 'item_list',
        '#list_type' => 'ul',
        '#items' => $summary,
      ];
    }

    return [
      'recipient_collector' => [
        '#markup' => $this->pluginDefinition['description'],
      ],
    ] + [
      'collector_extenders' => $collector_extenders_summary,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function performPostCollect(array &$recipients, MessageInterface $message): void {
    $extenders = $this->getExtenders();
    /** @var \Drupal\message_recipient\CollectorExtenderInterface $extender */
    foreach ($extenders as $extender) {
      if ($extender->getStage() === CollectorExtenderType::POST_COLLECT
        && $extender->isApplicable($message)) {
        $extender->postCollect($recipients, $message);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state, MessageTemplateInterface $message_template): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void {
  }

}
