<?php

declare(strict_types=1);

namespace Drupal\message_recipient;

use Drupal\Component\Plugin\ConfigurableInterface;
use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\message\MessageInterface;
use Drupal\message\MessageTemplateInterface;

/**
 * Interface for recipients collector instance.
 */
interface RecipientsCollectorInterface extends PluginInspectionInterface, ConfigurableInterface, ContainerFactoryPluginInterface {

  /**
   * Get the recipients of the message.
   *
   * @param \Drupal\message\MessageInterface $message
   *   The message entity.
   *
   * @return array
   *   The recipients' array.
   */
  public function collect(MessageInterface $message): array;

  /**
   * Check if whether the recipients' collector is applicable.
   *
   * @param \Drupal\message\MessageInterface $message
   *   The message entity.
   *
   * @return bool
   *   True if the recipients' collector is applicable, false otherwise.
   */
  public function isApplicable(MessageInterface $message): bool;

  /**
   * Set message template when initializing a recipients' collector plugin.
   *
   * @param \Drupal\message\MessageTemplateInterface $message_template
   *   The message template.
   *
   * @see \Drupal\message_recipient\RecipientsCollectorPluginCollection
   */
  public function initialize(MessageTemplateInterface $message_template): void;

  /**
   * Returns the unique ID representing the recipients' collector.
   *
   * @return string
   *   The recipients' collector uuid.
   */
  public function getUuid(): string;

  /**
   * Returns the recipients' collector label.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The recipients' collector label.
   */
  public function label(): TranslatableMarkup;

  /**
   * Returns the recipients' collector description.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The recipients' collector description.
   */
  public function description(): TranslatableMarkup;

  /**
   * Returns the recipients' collector data type.
   *
   * @return string
   *   The recipients' collector data type.
   */
  public function getType(): string;

  /**
   * Returns the recipients' collector category.
   *
   * @return string
   *   The recipients' collector category.
   */
  public function getCategory(): string;

  /**
   * Returns whether recipients' collector can be added multiple times.
   *
   * @return bool
   *   True when a recipients' collector can be added multiple times. False
   *   otherwise.
   */
  public function isMultipleAllowed(): bool;

  /**
   * Get data from configuration.
   *
   * @return array
   *   Data from configuration.
   */
  public function getData(): array;

  /**
   * Sets the extenders for this recipients' collector.
   *
   * @param array $configuration
   *   An associative array containing the extenders' configuration.
   *
   * @return self
   *   The recipients' collector.
   */
  public function setExtenders(array $configuration): self;

  /**
   * Adds extender this recipients' collector.
   *
   * @param string $extender_id
   *   The extender id.
   * @param array $settings
   *   The extender's configuration.
   *
   * @return self
   *   The recipients' collector.
   */
  public function addExtender(string $extender_id, array $settings = []): self;

  /**
   * Removes extender from this recipients' collector.
   *
   * @param string $extender_id
   *   The extender id.
   *
   * @return self
   *   The recipients' collector.
   */
  public function removeExtender(string $extender_id): self;

  /**
   * Gets extenders for this recipients' collector.
   *
   * @return \Drupal\message_recipient\CollectorExtenderPluginCollection|null
   *   The extenders, if any, null otherwise.
   */
  public function getExtenders(): ?CollectorExtenderPluginCollection;

  /**
   * Returns a render array summarizing the configuration.
   *
   * @return array
   *   A render array.
   */
  public function getSummary(): array;

  /**
   * Preform post-collect from configured collector extenders.
   *
   * @param array $recipients
   *   The array of recipients.
   * @param \Drupal\message\MessageInterface $message
   *   The message entity.
   */
  public function performPostCollect(array &$recipients, MessageInterface $message): void;

  /**
   * Adds a configuration form for this recipients' collector.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param \Drupal\message\MessageTemplateInterface $message_template
   *   The message template this recipients collector is being added to.
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state, MessageTemplateInterface $message_template): array;

  /**
   * Validates a configuration form for this recipients' collector.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param \Drupal\message\MessageTemplateInterface $message_template
   *   The message template this recipients collector is being added to.
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void;

  /**
   * Submits a configuration form for this recipients' collector.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param \Drupal\message\MessageTemplateInterface $message_template
   *   The message template this recipients collector is being added to.
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state, MessageTemplateInterface $message_template): void;

}
